module MDCOMPUTER #(
  parameter FREQUENCY = 12000000,  // Hz (1Hz-12MHz)
  parameter BITS      = 16         // 8 or 16
)(
  input  CLK,    // 16MHz clock
  output LED,    // User/boot LED next to power LED
  output USBPU,  // USB pull-up resistor
  output PIN_1,  // O[7]
  output PIN_2,  // O[6]
  output PIN_3,  // O[5]
  output PIN_4,  // O[4]
  output PIN_5,  // O[3]
  output PIN_6,  // O[2]
  output PIN_7,  // O[1]
  output PIN_8,  // O[0]
  input  PIN_9,  // reset
//  output PIN_10, // uart_tx output
  output PIN_14, // LCD rs
  output PIN_15, // LCD en
  output PIN_16, // LCD D0
  output PIN_17, // LCD D1
  output PIN_18, // LCD D2
  output PIN_19, // LCD D3
  output PIN_20, // LCD D4
  output PIN_21, // LCD D5
  output PIN_22, // LCD D6
  output PIN_23  // LCD D7
);
assign USBPU = 0; // drive USB pull-up resistor to '0' to disable USB

parameter integer FREQ = $floor(24000000 / $floor(24000000 / FREQUENCY));  // Hz

wire       clk , rst, lcd_rdy;//, lcd_clk;
wire       c_hlt,c_ce, c_co, c_j, c_mi, c_ii, c_ro, c_ri, c_ai, c_ao, c_bi, c_bo, c_di, c_do, c_oi, c_oo, c_eo, c_e0, c_e1, c_e2, c_ec, c_po;
wire [2:0] FLG;
wire [BITS-1:0] BUS, PC, MAR, RAM, ANS, A, B, D, O;
wire [7:0] IR;
reg        bus_active;

assign     rst  = 1'b0; //PIN_9;
assign     LED  = clk;

always @(c_co || c_ro || c_ao || c_bo || c_do || c_oo || c_eo) begin
  bus_active <= (c_co || c_ro || c_ao || c_bo || c_do || c_oo || c_eo);
end

CLOCK #(.CLK_FREQ(FREQ)) md_clock (
  .i_CLK        (CLK),
  .i_hlt        (c_hlt),
  .i_lcd_rdy    (lcd_rdy),
//  .o_lcd_clk    (lcd_clk),
  .o_clk        (clk)
);
//reg lcd_rdy = 1'b1;
LCD1602 #(.BITS(BITS),.CLK_FREQ(FREQ)) mylcd (
  .i_lcd_clk    (CLK),
  .i_o          (O),
  .o_rdy        (lcd_rdy),
  .o_en         (PIN_15),
  .o_lcd        ({PIN_14,PIN_23,PIN_22,PIN_21,PIN_20,PIN_19,PIN_18,PIN_17,PIN_16})
);

PROGRAM_COUNTER #(.BITS(BITS)) reg_PC (
  .i_clk        (clk),
  .i_rst        (rst),
  .i_co         (c_co),
  .i_ce         (c_ce),
  .i_j          (c_j),
  .io_bus       (BUS),
  .i_bus_active (bus_active),
  .o_pc         (PC)
);

REGISTER #(.BITS(BITS)) reg_MAR (
  .i_clk        (clk),
  .i_rst        (rst),
  .i_in         (c_mi),
  .i_out        (1'b0),
  .io_bus       (BUS),
  .i_bus_active (bus_active),
  .o_reg        (MAR)
);

RAM #(.BITS(BITS)) myram (
  .i_clk        (clk),
  .i_ri         (c_ri),
  .i_ro         (c_ro),
  .i_mar        (MAR),
  .io_bus       (BUS),
  .i_bus_active (bus_active),
  .o_ram        (RAM)
);

INSTRUCTION_REGISTER #(.BITS(BITS)) reg_IR (
  .i_clk        (clk),
  .i_rst        (rst),
  .i_lst        (1'b0),
  .i_prg        (1'b0),
  .i_flg        (FLG),
  .i_bus        (BUS),
  .i_bus_active (bus_active),
  .o_hlt        (c_hlt),
  .o_ce         (c_ce),
  .o_co         (c_co),
  .o_j          (c_j),
  .o_mi         (c_mi),
  .o_ii         (c_ii),
  .o_ro         (c_ro),
  .o_ri         (c_ri),
  .o_ai         (c_ai),
  .o_ao         (c_ao),
  .o_bi         (c_bi),
  .o_bo         (c_bo),
  .o_di         (c_di),
  .o_do         (c_do),
  .o_oi         (c_oi),
  .o_oo         (c_oo),
  .o_eo         (c_eo),
  .o_e0         (c_e0),
  .o_e1         (c_e1),
  .o_e2         (c_e2),
  .o_ec         (c_ec),
  .o_po         (c_po),
  .o_ir         (IR)
);

ALU #(.BITS(BITS)) reg_ALU (
  .i_clk        (clk),
  .i_rst        (rst),
  .i_eo         (c_eo),
  .i_e0         (c_e0),
  .i_e1         (c_e1),
  .i_e2         (c_e2),
  .i_ec         (c_ec),
  .i_rega       (A),
  .i_regb       (B),
  .o_bus        (BUS),
  .o_flg        (FLG),
  .o_ans        (ANS)
);

REGISTER #(.BITS(BITS)) reg_A (
  .i_clk        (clk),
  .i_rst        (rst),
  .i_in         (c_ai),
  .i_out        (c_ao),
  .io_bus       (BUS),
  .i_bus_active (bus_active),
  .o_reg        (A)
);

REGISTER #(.BITS(BITS)) reg_B (
  .i_clk        (clk),
  .i_rst        (rst),
  .i_in         (c_bi),
  .i_out        (c_bo),
  .io_bus       (BUS),
  .i_bus_active (bus_active),
  .o_reg        (B)
);

REGISTER #(.BITS(BITS)) reg_D (
  .i_clk        (clk),
  .i_rst        (rst),
  .i_in         (c_di),
  .i_out        (c_do),
  .io_bus       (BUS),
  .i_bus_active (bus_active),
  .o_reg        (D)
);

REGISTER #(.BITS(BITS)) reg_O (
  .i_clk        (clk),
  .i_rst        (rst),
  .i_in         (c_oi),
  .i_out        (c_oo),
  .io_bus       (BUS),
  .i_bus_active (bus_active),
  .o_reg        (O)
);
assign PIN_1 = O[7];
assign PIN_2 = O[6];
assign PIN_3 = O[5];
assign PIN_4 = O[4];
assign PIN_5 = O[3];
assign PIN_6 = O[2];
assign PIN_7 = O[1];
assign PIN_8 = O[0];
/*
wire       tx;
wire [7:0] uart_data = 8'b00101010;
reg [20:0] count;
always @(posedge CLK) begin
  count <= count + 1;
end
assign tx = (count == 0);
// 16MHz / 115k2 = 139
UART_TX #(.CLKS_PER_BIT(16000000/115200)) ser_tx (
  .i_Clock           (CLK),
  .i_Tx_DV           (tx),
  .i_Tx_Byte         (uart_data),
  .o_Tx_Serial       (PIN_10)
);
*/
endmodule
