#define       CLK               2
#define       SER               3
#define       LST               13
#define       PRG               14
#define       RO                15
#define       CO                16
#define       RI                4
#define       B0                5
#define       B1                6
#define       B2                7
#define       B3                8
#define       B4                9
#define       B5                10
#define       B6                11
#define       B7                12

String inputString = "";         // a string to hold incoming data
int serMode,ioMode,lstMode,prgMode,loadMode;

void setup() {
  Serial.begin(115200);
  inputString.reserve(770);
  pinMode(SER, INPUT);
  pinMode(CLK, INPUT);
  pinMode(RI, INPUT);
  pinMode(RO, INPUT);
  pinMode(CO, INPUT);
  setSerial();
  while (Serial.available())
    char inChar = (char)Serial.read();
  Serial.println("md8-Programmer");
  Serial.println("Send 'L' to toggle LST-mode");
  Serial.println("Send 'P' to toggle PRG-mode");
  loadMode = serMode;
  if (loadMode) {
    inputString = "2A 00 4A 58 2A 01 4A 54 4A 55 4A 56 4A 57 38 2A "
                  "02 4A 59 EA 32 51 02 91 4B 38 2A 00 4A 55 1A 55 "
                  "56 4A 55 1A 54 70 55 90 3B 2A 59 50 55 4A 56 AB "
                  "56 4B 57 32 6B 90 14 91 34 99 1E 2A 59 50 54 4A "
                  "55 1A 54 56 4A 54 ED BD 55 99 14 1A 58 56 4A 58 "
                  "7F 93 04 FF 01 01 01 01 00 02";
    setLstMode(1);
  }
  attachInterrupt(digitalPinToInterrupt(SER), handleInterruptSER, CHANGE);    
  Serial.println("READY");
}

void loop() {
    while (serMode && !loadMode && Serial.available()) {
    char inChar = (char)Serial.read();
    if (inChar >= 'a' && inChar <= 'z')
      inChar = inChar - 'a' + 'A';
    if (inChar >= '0' && inChar <= 'F')
      inputString += inChar;
    else if (inChar == 'L') {
      setLstMode(1 - lstMode);
      if (!lstMode && prgMode)
      setPrgMode(0);
      Serial.print("\r\nLST=");
      Serial.print(lstMode);
      Serial.print(" PRG=");
      Serial.println(prgMode);
    }
    else if (inChar == 'P') {
      setPrgMode(1 - prgMode);
      if (prgMode && !lstMode)
        setLstMode(1);
      Serial.print("\r\nLST=");
      Serial.print(lstMode);
      Serial.print(" PRG=");
      Serial.println(prgMode);
    }
  }
}

void handleInterruptCLK() {
  if (lstMode) {
    if (prgMode && digitalRead(RI) && !digitalRead(RO))
      setData(OUTPUT);
    else
      setData(INPUT);
  }
}

void handleInterruptSER() {
  setSerial();
}

void setSerial() {
  int ENABLED = digitalRead(SER);
  if (serMode != ENABLED) {
    serMode = ENABLED;
    loadMode = 0;
    setData(INPUT);
    if (ENABLED) {
      pinMode(LST, OUTPUT);
      setLstMode(0);
      pinMode(PRG, OUTPUT);
      setPrgMode(0);
      attachInterrupt(digitalPinToInterrupt(CLK), handleInterruptCLK, FALLING);    
    }
    else {
      setLstMode(0);
      setPrgMode(0);
      detachInterrupt(digitalPinToInterrupt(CLK));    
      pinMode(LST, INPUT);
      pinMode(PRG, INPUT);
    }
  }
}

void setData (int inout) {
  if (ioMode != inout) {
    ioMode = inout;
    pinMode(B0, inout);
    pinMode(B1, inout);
    pinMode(B2, inout);
    pinMode(B3, inout);
    pinMode(B4, inout);
    pinMode(B5, inout);
    pinMode(B6, inout);
    pinMode(B7, inout);
  }
  if (ioMode == OUTPUT) {
    // OUTPUT case
    byte data;
    while (inputString.length() > 0 && (inputString.substring(0,1) < "0" || inputString.substring(0,1) > "F"))
      inputString.remove(0, 1);
    if (inputString.length() < 2) {
      data = 0x00;
    }
    else {
      data = hex2byte(inputString.substring(0,2));
      inputString.remove(0, 2);
    }
    writeData(data);
    Serial.print(String(data, HEX) + " ");
  }
  // INPUT cases
  else if (lstMode && digitalRead(RO) && !digitalRead(RI)) {                // RAM list
    Serial.print(String(readData(), HEX) + " ");
  }
  else if (loadMode && lstMode && digitalRead(CO) && readData() == 0x0) {   // loadMode, ADR=0x0
    if (!prgMode) {                                                         // Start programming
      setPrgMode(1);
    }
    else {                                                                  // Stop loadMode
      loadMode = 0;
      setPrgMode(0);
      setLstMode(0);
    }
  }
}

void setLstMode (int onoff) {
  lstMode = onoff;
  digitalWrite(LST,lstMode);
}

void setPrgMode (int onoff) {
  prgMode = onoff;
  digitalWrite(PRG,prgMode);
}

byte hex2byte(String hs){
  char c[hs.length() + 1];
  hs.toCharArray(c, hs.length() + 1);
  return (byte)strtol(c, NULL, 16); 
}

void writeData(byte data) {
  digitalWrite(B0, (data & 0b00000001) != 0);
  digitalWrite(B1, (data & 0b00000010) != 0);
  digitalWrite(B2, (data & 0b00000100) != 0);
  digitalWrite(B3, (data & 0b00001000) != 0);
  digitalWrite(B4, (data & 0b00010000) != 0);
  digitalWrite(B5, (data & 0b00100000) != 0);
  digitalWrite(B6, (data & 0b01000000) != 0);
  digitalWrite(B7, (data & 0b10000000) != 0);
}

byte readData() {
  return digitalRead(B0)
       + digitalRead(B1) * 2
       + digitalRead(B2) * 4
       + digitalRead(B3) * 8
       + digitalRead(B4) * 16
       + digitalRead(B5) * 32
       + digitalRead(B6) * 64
       + digitalRead(B7) * 128;  
}

